"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.getAutoTaggingComponent = exports.entityGroups = void 0;
const React = require("react");
const immutable_1 = require("immutable");
const react_1 = require("superdesk-ui-framework/react");
const superdesk_ui_framework_1 = require("superdesk-ui-framework");
const tag_list_1 = require("./tag-list");
const new_item_1 = require("./new-item");
const adapter_1 = require("./adapter");
const constants_1 = require("./constants");
const groups_1 = require("./groups");
const common_1 = require("./common");
const data_transformations_1 = require("./data-transformations");
const lodash_1 = require("lodash");
exports.entityGroups = immutable_1.OrderedSet(['place', 'person', 'organisation']);
const RUN_AUTOMATICALLY_PREFERENCE = 'run_automatically';
function tagAlreadyExists(data, qcode) {
    return data.changes.analysis.has(qcode);
}
function hasConfig(key, iMatricsFields) {
    return iMatricsFields[key] != null;
}
function getAutoTaggingComponent(superdesk, label) {
    const { preferences } = superdesk;
    const { httpRequestJsonLocal } = superdesk;
    const { gettext } = superdesk.localization;
    const { memoize, generatePatch } = superdesk.utilities;
    const groupLabels = groups_1.getGroups(superdesk);
    const TagListComponent = tag_list_1.getTagsListComponent(superdesk);
    const NewItemComponent = new_item_1.getNewItemComponent(superdesk);
    return class AutoTagging extends React.PureComponent {
        constructor(props) {
            var _a;
            super(props);
            this.iMatricsFields = (_a = superdesk.instance.config.iMatricsFields) !== null && _a !== void 0 ? _a : { entities: {}, others: {} };
            this.state = {
                data: 'not-initialized',
                newItem: null,
                runAutomaticallyPreference: 'loading',
                vocabularyLabels: null,
            };
            this._mounted = false;
            this.runAnalysis = this.runAnalysis.bind(this);
            this.initializeData = this.initializeData.bind(this);
            this.updateTags = this.updateTags.bind(this);
            this.createNewTag = this.createNewTag.bind(this);
            this.insertTagFromSearch = this.insertTagFromSearch.bind(this);
            this.reload = this.reload.bind(this);
            this.isDirty = memoize((a, b) => Object.keys(generatePatch(a, b)).length > 0);
        }
        runAnalysis() {
            const dataBeforeLoading = this.state.data;
            this.setState({ data: 'loading' }, () => {
                const { guid, language, headline, body_html, abstract } = this.props.article;
                httpRequestJsonLocal({
                    method: 'POST',
                    path: '/ai/',
                    payload: {
                        service: 'imatrics',
                        item: {
                            guid,
                            language,
                            headline,
                            body_html,
                            abstract,
                        },
                    },
                }).then((res) => {
                    const resClient = adapter_1.toClientFormat(res.analysis);
                    if (this._mounted) {
                        this.setState({
                            data: {
                                original: dataBeforeLoading === 'loading' || dataBeforeLoading === 'not-initialized'
                                    ? { analysis: immutable_1.OrderedMap() } // initialize empty data
                                    : dataBeforeLoading.original,
                                changes: { analysis: resClient },
                            },
                        });
                    }
                });
            });
        }
        initializeData(preload) {
            const existingTags = data_transformations_1.getExistingTags(this.props.article);
            if (Object.keys(existingTags).length > 0) {
                const resClient = adapter_1.toClientFormat(existingTags);
                this.setState({
                    data: { original: { analysis: resClient }, changes: { analysis: resClient } },
                });
            }
            else if (preload) {
                this.runAnalysis();
            }
        }
        updateTags(tags, data) {
            const { changes } = data;
            this.setState({
                data: Object.assign(Object.assign({}, data), { changes: Object.assign(Object.assign({}, changes), { analysis: tags }) }),
            });
        }
        createNewTag(newItem, data) {
            const _title = newItem.name;
            if (_title == null || newItem.group == null) {
                return;
            }
            const tag = {
                qcode: Math.random().toString(),
                name: _title,
                description: newItem.description,
                source: constants_1.SOURCE_IMATRICS,
                altids: {},
                group: newItem.group,
            };
            this.updateTags(data.changes.analysis.set(tag.qcode, tag), data);
            this.setState({ newItem: null });
        }
        insertTagFromSearch(tag, data) {
            this.updateTags(data.changes.analysis.set(tag.qcode, tag), data);
        }
        getGroupName(group, vocabularyLabels) {
            var _a, _b, _c;
            return (_c = (_b = (_a = this.iMatricsFields.others[group]) === null || _a === void 0 ? void 0 : _a.name) !== null && _b !== void 0 ? _b : vocabularyLabels === null || vocabularyLabels === void 0 ? void 0 : vocabularyLabels.get(group)) !== null && _c !== void 0 ? _c : group;
        }
        reload() {
            this.setState({ data: 'not-initialized' });
            this.initializeData(false);
        }
        componentDidMount() {
            this._mounted = true;
            Promise.all([
                common_1.getAutoTaggingVocabularyLabels(superdesk),
                preferences.get(RUN_AUTOMATICALLY_PREFERENCE),
            ]).then(([vocabularyLabels, runAutomatically = false]) => {
                this.setState({
                    vocabularyLabels,
                    runAutomaticallyPreference: runAutomatically,
                });
                this.initializeData(runAutomatically);
            });
        }
        componentWillUnmount() {
            this._mounted = false;
        }
        render() {
            const { runAutomaticallyPreference, vocabularyLabels } = this.state;
            if (runAutomaticallyPreference === 'loading' || vocabularyLabels == null) {
                return null;
            }
            const { data } = this.state;
            const dirty = data === 'loading' || data === 'not-initialized' ? false :
                this.isDirty(data.original, data.changes);
            const readOnly = superdesk.entities.article.isLockedInOtherSession(this.props.article);
            return (React.createElement(React.Fragment, null,
                React.createElement("div", { className: "widget-header" },
                    React.createElement("div", { className: "widget-title" }, label),
                    data === 'loading' || data === 'not-initialized' || !dirty ? null : (React.createElement("div", { className: "widget__sliding-toolbar widget__sliding-toolbar--right" },
                        React.createElement("button", { className: "btn btn--primary", onClick: () => {
                                superdesk.entities.article.patch(this.props.article, data_transformations_1.createTagsPatch(this.props.article, data.changes.analysis, superdesk)).then(() => {
                                    this.reload();
                                });
                            } }, gettext('Save')),
                        React.createElement("button", { className: "btn", onClick: () => {
                                this.reload();
                            } }, gettext('Cancel'))))),
                React.createElement("div", { className: "widget-content sd-padding-all--2" },
                    React.createElement("div", null,
                        React.createElement("div", { className: "form__row form__row--flex", style: { padding: 0 } },
                            React.createElement(react_1.ButtonGroup, { align: "left" },
                                React.createElement(react_1.Switch, { value: runAutomaticallyPreference, disabled: readOnly, onChange: () => {
                                        const newValue = !runAutomaticallyPreference;
                                        this.setState({ runAutomaticallyPreference: newValue });
                                        superdesk.preferences.set(RUN_AUTOMATICALLY_PREFERENCE, newValue);
                                        if (newValue && this.state.data === 'not-initialized') {
                                            this.runAnalysis();
                                        }
                                    } }),
                                React.createElement("label", null, gettext('Run automatically')))),
                        data === 'loading' || data === 'not-initialized' ? null : (React.createElement("div", { className: "form__row form__row--flex", style: { alignItems: 'center' } },
                            React.createElement("div", { style: { flexGrow: 1 } },
                                React.createElement(react_1.Autocomplete, { value: '', keyValue: "name", items: [], search: (searchString, callback) => {
                                        let cancelled = false;
                                        httpRequestJsonLocal({
                                            method: 'POST',
                                            path: '/ai_data_op/',
                                            payload: {
                                                service: 'imatrics',
                                                operation: 'search',
                                                data: { term: searchString },
                                            },
                                        }).then((res) => {
                                            if (cancelled !== true) {
                                                const result = adapter_1.toClientFormat(res.result.tags).toArray();
                                                const withoutExistingTags = result.filter((searchTag) => tagAlreadyExists(data, searchTag.qcode) !== true);
                                                callback(withoutExistingTags);
                                            }
                                        });
                                        return {
                                            cancel: () => {
                                                cancelled = true;
                                            },
                                        };
                                    }, listItemTemplate: (__item) => {
                                        var _a;
                                        const _item = __item;
                                        return (React.createElement("div", { className: "auto-tagging-widget__autocomplete-item" },
                                            React.createElement("b", null, _item.name),
                                            ((_a = _item === null || _item === void 0 ? void 0 : _item.group) === null || _a === void 0 ? void 0 : _a.value) == null ? null : (React.createElement("p", null, _item.group.value)),
                                            (_item === null || _item === void 0 ? void 0 : _item.description) == null ? null : (React.createElement("p", null, _item.description))));
                                    }, onSelect: (_value) => {
                                        const value = _value;
                                        this.insertTagFromSearch(value, data);
                                        // TODO: clear autocomplete?
                                    }, onChange: lodash_1.noop })),
                            React.createElement("div", { style: { marginLeft: 10 } },
                                React.createElement(react_1.Button, { type: "primary", icon: "plus-large", size: "small", shape: "round", text: gettext('Add'), iconOnly: true, disabled: readOnly, onClick: () => {
                                        this.setState({
                                            newItem: {
                                                name: '',
                                            },
                                        });
                                    } }))))),
                    (() => {
                        if (data === 'loading') {
                            return (React.createElement("div", { style: { display: 'flex', alignItems: 'center' } },
                                React.createElement("div", { className: "spinner-big" })));
                        }
                        else if (data === 'not-initialized') {
                            return (React.createElement(react_1.EmptyState, { title: gettext('No tags yet'), description: readOnly ? undefined : gettext('Click "Run" to generate') }));
                        }
                        else {
                            const items = data.changes.analysis;
                            const savedTags = data.original.analysis.keySeq().toSet();
                            const isEntity = (tag) => exports.entityGroups.has(tag.group.value);
                            const entities = items.filter((tag) => isEntity(tag));
                            const entitiesGrouped = entities.groupBy((tag) => tag === null || tag === void 0 ? void 0 : tag.group.value);
                            const entitiesGroupedAndSortedByConfig = entitiesGrouped
                                .filter((_, key) => hasConfig(key, this.iMatricsFields.entities))
                                .sortBy((_, key) => this.iMatricsFields.entities[key].order, (a, b) => a - b);
                            const entitiesGroupedAndSortedNotInConfig = entitiesGrouped
                                .filter((_, key) => !hasConfig(key, this.iMatricsFields.entities))
                                .sortBy((_, key) => key.toString().toLocaleLowerCase(), (a, b) => a.localeCompare(b));
                            const entitiesGroupedAndSorted = entitiesGroupedAndSortedByConfig
                                .concat(entitiesGroupedAndSortedNotInConfig);
                            const others = items.filter((tag) => isEntity(tag) === false);
                            const othersGrouped = others.groupBy((tag) => tag.group.value);
                            let allGrouped = immutable_1.OrderedMap();
                            othersGrouped.forEach((tags, groupId) => tags && groupId ? allGrouped = allGrouped.set(groupId, React.createElement(superdesk_ui_framework_1.ToggleBoxNext, { key: groupId, title: this.getGroupName(groupId, vocabularyLabels), style: "circle", isOpen: true },
                                React.createElement(TagListComponent, { savedTags: savedTags, tags: tags.toMap(), readOnly: readOnly, onRemove: (id) => {
                                        this.updateTags(data.changes.analysis.remove(id), data);
                                    } }))) : false);
                            if (entitiesGroupedAndSorted.size > 0) {
                                allGrouped = allGrouped.set('entities', React.createElement(superdesk_ui_framework_1.ToggleBoxNext, { title: this.getGroupName('entities', vocabularyLabels), style: "circle", isOpen: true, key: "entities" }, entitiesGroupedAndSorted.map((tags, key) => (React.createElement("div", { key: key },
                                    React.createElement("div", { className: "form-label", style: { display: 'block' } }, groupLabels.get(key).plural),
                                    React.createElement(TagListComponent, { savedTags: savedTags, tags: tags.toMap(), readOnly: readOnly, onRemove: (id) => {
                                            this.updateTags(data.changes.analysis.remove(id), data);
                                        } })))).toArray()));
                            }
                            const allGroupedAndSortedByConfig = allGrouped
                                .filter((_, key) => hasConfig(key, this.iMatricsFields.others))
                                .sortBy((_, key) => this.iMatricsFields.others[key].order, (a, b) => a - b);
                            const allGroupedAndSortedNotInConfig = allGrouped
                                .filter((_, key) => !hasConfig(key, this.iMatricsFields.others));
                            const allGroupedAndSorted = allGroupedAndSortedByConfig
                                .concat(allGroupedAndSortedNotInConfig);
                            return (React.createElement(React.Fragment, null,
                                this.state.newItem == null ? null : (React.createElement(NewItemComponent, { item: this.state.newItem, onChange: (newItem) => {
                                        this.setState({ newItem });
                                    }, save: (newItem) => {
                                        this.createNewTag(newItem, data);
                                    }, cancel: () => {
                                        this.setState({ newItem: null });
                                    }, tagAlreadyExists: (qcode) => tagAlreadyExists(data, qcode), insertTagFromSearch: (tag) => {
                                        this.insertTagFromSearch(tag, data);
                                    } })),
                                React.createElement("div", { className: "widget-content__main" }, allGroupedAndSorted.map((item) => item).toArray())));
                        }
                    })(),
                    React.createElement("div", { className: "widget-content__footer" }, (() => {
                        if (data === 'loading') {
                            return null;
                        }
                        else if (data === 'not-initialized') {
                            return (React.createElement(react_1.Button, { type: "primary", text: gettext('Run'), expand: true, disabled: readOnly, onClick: () => {
                                    this.runAnalysis();
                                } }));
                        }
                        else {
                            return (React.createElement(react_1.Button, { type: "primary", text: gettext('Refresh'), expand: true, disabled: readOnly, onClick: () => {
                                    this.runAnalysis();
                                } }));
                        }
                    })()))));
        }
    };
}
exports.getAutoTaggingComponent = getAutoTaggingComponent;
