"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const React = require("react");
const lodash_1 = require("lodash");
const initialState = { type: null, value: 0, rotateDegree: 0, scale: 1 };
class VideoEditorThumbnail extends React.Component {
    constructor(props) {
        super(props);
        this.state = initialState;
        this.ref = React.createRef();
        this.maxCanvasSize = { width: 0, height: 0 };
        this.interval = 0;
        this.handleClick = this.handleClick.bind(this);
        this.handleUpload = this.handleUpload.bind(this);
        this.handleSave = this.handleSave.bind(this);
        this.handleReset = this.handleReset.bind(this);
        this.getWrapperSize = this.getWrapperSize.bind(this);
    }
    componentDidMount() {
        var _a, _b;
        const thumbnail = (_b = (_a = this.props.article.renditions) === null || _a === void 0 ? void 0 : _a.viewImage) === null || _b === void 0 ? void 0 : _b.href;
        if (thumbnail) {
            this.setThumbnail(thumbnail);
        }
    }
    componentDidUpdate(prevProps) {
        // video has changed so captured thumbnail showed on UI is no longer correct
        if (prevProps.article.renditions !== this.props.article.renditions && this.state.type === 'capture') {
            this.handleReset();
        }
    }
    componentWillUnmount() {
        if (this.interval) {
            clearInterval(this.interval);
        }
    }
    handleClick() {
        this.setState({
            type: 'capture',
            value: this.props.video.currentTime,
            rotateDegree: this.props.rotate,
        }, () => {
            // make sure rotateDegree is updated so we can calculate scale ratio correctly
            const video = this.props.video;
            let { x, y, width, height, aspect } = this.props.getCropRotate(this.props.crop);
            aspect = (aspect !== null && aspect !== void 0 ? aspect : 1);
            let canvasSize = [this.maxCanvasSize.width, this.maxCanvasSize.height];
            // crop is disabled or has not drew crop area yet
            const isDisabledCrop = [x, y, width, height].every((value) => value === 0);
            if (isDisabledCrop) {
                aspect = video.videoWidth / video.videoHeight;
            }
            if (this.state.rotateDegree % 180 !== 0) {
                if (!isDisabledCrop) {
                    aspect = 1 / aspect;
                }
                // make thumbnail overflow then scale it down, otherwise thumbnail will be too small
                // once rotated because we draw thumbnail based on canvas width
                canvasSize = [this.maxCanvasSize.height, this.maxCanvasSize.width];
            }
            this.drawCanvas(video, x || 0, y || 0, width || video.videoWidth, height || video.videoHeight, aspect, canvasSize);
        });
    }
    handleUpload(files) {
        var _a;
        const file = (_a = files) === null || _a === void 0 ? void 0 : _a[0];
        if (file == null) {
            return;
        }
        const reader = new FileReader();
        reader.onload = () => {
            if (typeof reader.result === 'string') {
                this.setThumbnail(reader.result);
            }
        };
        reader.readAsDataURL(file);
        this.setState(Object.assign(Object.assign({}, initialState), { value: file, type: 'upload' }));
    }
    handleSave() {
        const { gettext } = this.props.superdesk.localization;
        const { session, instance } = this.props.superdesk;
        const host = instance.config.server.url;
        if (this.state.type === 'capture' && typeof this.state.value === 'number') {
            const crop = this.props.getCropRotate(lodash_1.pick(this.props.crop, ['x', 'y', 'width', 'height']));
            const body = {
                // Captured thumbnail from server and from canvas have small difference in time (position)
                position: this.state.value - 0.04,
                crop: Object.values(crop).join(','),
                rotate: this.state.rotateDegree,
            };
            if (body.crop === '0,0,0,0') {
                delete body.crop;
            }
            if (body.rotate === 0) {
                delete body.rotate;
            }
            // dataApi.create requires both body and response is the same type
            fetch(`${host}/video_edit`, {
                method: 'POST',
                headers: {
                    Authorization: session.getToken(),
                    'Content-Type': 'application/json',
                    'If-Match': this.props.article._etag,
                },
                body: JSON.stringify({
                    capture: body,
                    item: {
                        _id: this.props.article._id,
                        renditions: this.props.article.renditions,
                    },
                }),
            })
                .then(() => {
                // reuse thumbnail from canvas so we don't have to display the old one,
                // new thumbnail will be loaded when user reset changes
                this.setState(Object.assign(Object.assign({}, initialState), { scale: this.state.scale, rotateDegree: this.props.rotate }));
                this.props.onToggleLoading(true, gettext('Saving capture thumbnail...'), 'thumbnail');
                this.getThumbnail();
            })
                .catch(this.props.onError);
        }
        else if (this.state.type === 'upload' && this.state.value instanceof File) {
            const form = new FormData();
            form.append('file', this.state.value);
            fetch(`${host}/video_edit/${this.props.article._id}`, {
                method: 'PUT',
                headers: {
                    Authorization: session.getToken(),
                    'If-Match': this.props.article._etag,
                },
                body: form,
            })
                .then((res) => res.json())
                .then((res) => {
                var _a, _b, _c;
                this.handleReset();
                this.props.onSave(res.renditions, res._etag);
                this.setThumbnail((_c = (_b = (_a = res.renditions) === null || _a === void 0 ? void 0 : _a.viewImage) === null || _b === void 0 ? void 0 : _b.href, (_c !== null && _c !== void 0 ? _c : '')));
            })
                .catch(this.props.onError);
        }
    }
    handleReset() {
        this.clearCanvas();
        this.setState(Object.assign(Object.assign({}, initialState), { scale: this.state.scale }));
    }
    setThumbnail(src) {
        const image = new Image();
        image.onload = () => {
            this.drawCanvas(image, 0, 0, image.width, image.height);
        };
        image.src = src;
        // file reader result
        if (src.startsWith('data:') === false) {
            // Firefox does not reload image even no-store cache-control header is set
            image.src = src + `?t=${Math.random()}`;
        }
    }
    getThumbnail() {
        const { dataApi } = this.props.superdesk;
        this.interval = window.setInterval(() => {
            dataApi
                .findOne('video_edit', this.props.article._id)
                .then((response) => {
                var _a, _b;
                if (((_b = (_a = response.project) === null || _a === void 0 ? void 0 : _a.processing) === null || _b === void 0 ? void 0 : _b.thumbnail_preview) === false) {
                    clearInterval(this.interval);
                    this.props.onSave(response.renditions, response._etag);
                    this.props.onToggleLoading(false, '', 'thumbnail');
                }
            })
                .catch((_) => {
                clearInterval(this.interval);
            });
        }, 1500);
    }
    drawCanvas(element, x, y, width, height, ratio = width / height, canvasSize = [this.maxCanvasSize.width, this.maxCanvasSize.height]) {
        var _a;
        if (this.ref.current == null) {
            throw new Error('Could not get current canvas');
        }
        let [drawWidth, drawHeight] = canvasSize;
        if (ratio > 1) {
            drawHeight = drawWidth / ratio;
        }
        else {
            drawWidth = drawHeight * ratio;
        }
        this.ref.current.width = drawWidth;
        this.ref.current.height = drawHeight;
        (_a = this.ref.current.getContext('2d')) === null || _a === void 0 ? void 0 : _a.drawImage(element, x, y, width, height, 0, 0, drawWidth, drawHeight);
        this.setScale(this.ref.current);
    }
    clearCanvas() {
        var _a, _b, _c;
        const ctx = (_a = this.ref.current) === null || _a === void 0 ? void 0 : _a.getContext('2d');
        if (this.ref.current == null || ctx == null) {
            throw new Error('Could not get current canvas');
        }
        ctx.clearRect(0, 0, this.ref.current.width, this.ref.current.height);
        const thumbnail = (_c = (_b = this.props.article.renditions) === null || _b === void 0 ? void 0 : _b.viewImage) === null || _c === void 0 ? void 0 : _c.href;
        if (thumbnail) {
            this.setThumbnail(thumbnail);
        }
    }
    // get wrapper size dynamically to use for calculating canvas size to fit content into
    getWrapperSize(element) {
        if (element == null) {
            return;
        }
        const { width, height } = element.getBoundingClientRect();
        this.maxCanvasSize = {
            width: width,
            height: height,
        };
    }
    setScale(ref) {
        // calculate scale while rotating to make sure image is not exceeded maximum wrapper size
        let scale = 1;
        const height = ref.getBoundingClientRect().height / this.state.scale;
        if (height > this.maxCanvasSize.height) {
            scale = this.maxCanvasSize.height / height;
        }
        else if (height === this.maxCanvasSize.height && this.state.scale < 1) {
            // scale was calculated on previous call
            return;
        }
        this.setState({ scale: scale });
    }
    render() {
        var _a, _b;
        const { getClass } = this.props.superdesk.utilities.CSS;
        const { gettext } = this.props.superdesk.localization;
        return (React.createElement("div", { className: `sd-photo-preview__thumbnail-edit ${getClass('thumbnail__container')}` },
            React.createElement("div", { className: "sd-photo-preview__thumbnail-edit-label" }, gettext('Video thumbnail')),
            React.createElement("div", { className: "image-overlay" },
                React.createElement("div", { className: "image-overlay__button-block" }, this.state.type === null ? (React.createElement(React.Fragment, null,
                    React.createElement("a", { className: "image-overlay__button", "sd-tooltip": gettext('Use current frame'), onClick: this.handleClick },
                        React.createElement("i", { className: "icon-photo" })),
                    React.createElement("form", { className: "sd-margin-l--1" },
                        React.createElement("label", null,
                            React.createElement("input", { type: "file", style: { display: 'none' }, accept: ".png,.jpg,.jpeg,.webp", onChange: (e) => this.handleUpload(e.target.files) }),
                            React.createElement("a", { className: "image-overlay__button", "sd-tooltip": gettext('Upload image') },
                                React.createElement("i", { className: "icon-upload" })))))) : (React.createElement(React.Fragment, null,
                    React.createElement("a", { className: "image-overlay__button", "sd-tooltip": gettext('Save change'), onClick: this.handleSave },
                        React.createElement("i", { className: "icon-ok" })),
                    React.createElement("a", { className: "image-overlay__button", "sd-tooltip": gettext('Reset change'), onClick: this.handleReset },
                        React.createElement("i", { className: "icon-close-thick" })))))),
            !((_b = (_a = this.props.article.renditions) === null || _a === void 0 ? void 0 : _a.viewImage) === null || _b === void 0 ? void 0 : _b.href) && !this.state.value && (React.createElement("div", { className: getClass('thumbnail--empty') },
                React.createElement("div", { className: "upload__info-icon" }),
                React.createElement("p", { className: getClass('thumbnail--empty__text') }, gettext('No thumbnail')))),
            React.createElement("div", { className: getClass('thumbnail__wrapper'), ref: this.getWrapperSize },
                React.createElement("canvas", { ref: this.ref, style: { transform: `rotate(${this.state.rotateDegree}deg) scale(${this.state.scale})` } }))));
    }
}
exports.VideoEditorThumbnail = VideoEditorThumbnail;
